@@ -148,9 +148,12 @@ if (getRversion() >= "3.1.0") {
 #'                  This is optional if \code{Cache} is used inside a SpaDES module.
 #'
 #' @param length Numeric. If the element passed to Cache is a \code{Path} class
-#'        object (from e.g., \code{asPath(filename)}) or it is a \code{Raster} with file-backing, then this will be
+#'        object (from e.g., \code{asPath(filename)}) or it is a \code{Raster} with
+#'        file-backing, then this will be
 #'        passed to \code{digest::digest}, essentially limiting the number of bytes
 #'        to digest (for speed). This will only be used if \code{quick = FALSE}.
+#'        Default is \code{getOption("reproducible.length")},
+#'        which is set to \code{Inf}.
 #'
 #' @param compareRasterFileLength Being deprecated; use \code{length}.
 #'
@@ -213,12 +216,17 @@ if (getRversion() >= "3.1.0") {
 #'        where Cache is not correctly detecting unchanged inputs. This will guarantee
 #'        the object will be identical each time; this may be useful in operational code.
 #'
-#' @param useCache Logical. If \code{FALSE}, then the entire Caching mechanism is bypassed
+#' @param useCache Logical or \code{"overwrite"}. If \code{FALSE},
+#'                 then the entire Caching mechanism is bypassed
 #'                 and the function is evaluated as if it was not being Cached.
 #'                 Default is \code{getOption("reproducible.useCache")}),
 #'                 which is \code{FALSE} by default, meaning use the Cache mechanism. This
 #'                 may be useful to turn all Caching on or off in very complex scripts and
-#'                 nested functions.
+#'                 nested functions. If \code{"overwrite"} (which can be set with
+#'                 \code{options("reproducible.useCache" = "overwrite")}),
+#'                 then the function invoke the caching mechanism but will purge
+#'                 any entry that is matched, and it will be replaced with the
+#'                 results of the current call.
 #'
 #' @param showSimilar A logical or numeric. Useful for debugging.
 #'        If \code{TRUE} or \code{1}, then if the Cache
@@ -270,7 +278,8 @@ if (getRversion() >= "3.1.0") {
 setGeneric(
   "Cache", signature = "...",
   function(FUN, ..., notOlderThan = NULL, objects = NULL, outputObjects = NULL, # nolint
-           algo = "xxhash64", cacheRepo = NULL, length = 1e6,
+           algo = "xxhash64", cacheRepo = NULL,
+           length = getOption("reproducible.length", Inf),
            compareRasterFileLength, userTags = c(),
            digestPathContent, omitArgs = NULL,
            classOptions = list(), debugCache = character(),
@@ -302,7 +311,7 @@ setMethod(
     modifiedDots <- fnDetails$modifiedDots
     originalDots <- fnDetails$originalDots
 
-    if (!useCache) {
+    if (isFALSE(useCache)) {
       message(crayon::green("useCache is FALSE, skipping Cache.",
                             "To turn Caching on, use options(reproducible.useCache = TRUE)"))
       if (fnDetails$isDoCall) {
@@ -494,7 +503,6 @@ setMethod(
       # don't digest the dotPipe elements as they are already
       # extracted individually into modifiedDots list elements
       dotPipe <- startsWith(names(modifiedDots), "._")
-
       preDigestByClass <- lapply(seq_along(modifiedDots[!dotPipe]), function(x) {
         .preDigestByClass(modifiedDots[!dotPipe][[x]])
       })
@@ -502,7 +510,13 @@ setMethod(
       if (verbose) {
         startHashTime <- Sys.time()
       }
-      preDigest <- lapply(modifiedDots[!dotPipe], function(x) {
+
+      # remove some of the arguments passed to Cache, which are irrelevant for digest
+      argsToOmitForDigest <- dotPipe |
+        (names(modifiedDots) %in%
+           c("debug", "notOlderThan", "debugCache", "verbose", "useCache", "showSimilar"))
+
+      preDigest <- lapply(modifiedDots[!argsToOmitForDigest], function(x) {
         # remove the "newCache" attribute, which is irrelevant for digest
         if (!is.null(attr(x, ".Cache")$newCache)) attr(x, ".Cache")$newCache <- NULL
         .robustDigest(x, objects = objects,
@@ -607,7 +621,19 @@ setMethod(
         }
       }
 
+      if (length(debugCache)) {
+        if (!is.na(pmatch(debugCache, "iterative")))
+          browser()
+      }
+
       isInRepo <- localTags[localTags$tag == paste0("cacheId:", outputHash), , drop = FALSE]
+      if (identical("overwrite", useCache) && NROW(isInRepo)>0) {
+        clearCache(x = cacheRepo, userTags = outputHash, ask = FALSE)
+        isInRepo <- isInRepo[isInRepo$tag != paste0("cacheId:", outputHash), , drop = FALSE]
+        message("Overwriting Cache entry with function '",fnDetails$functionName ,"'")
+
+      }
+
       # If it is in the existing record:
 
       if (NROW(isInRepo) > 0) {
@@ -640,6 +666,7 @@ setMethod(
                                         repoDir = cacheRepo, value = TRUE)
           }
 
+
           if (verbose) {
             endLoadTime <- Sys.time()
             verboseDF <- data.frame(
@@ -928,6 +955,12 @@ setMethod(
       # extract other function names that are not the ones the focus of the Cache call
       otherFns <- .getOtherFnNamesAndTags(scalls = scalls)
 
+      # Remove from otherFunctions if it is "function"
+      alreadyIn <- gsub(otherFns, pattern = "otherFunctions:", replacement = "") %in%
+        as.character(attr(output, "function"))
+      if (isTRUE(any(alreadyIn)))
+        otherFns <- otherFns[!alreadyIn]
+
       outputToSaveIsList <- is.list(outputToSave)
       if (outputToSaveIsList) {
         rasters <- unlist(lapply(outputToSave, is, "Raster"))
@@ -1206,9 +1239,14 @@ showLocalRepo3Mem <- memoise::memoise(showLocalRepo3)
 #' @param userTags Character string of tags to attach to this \code{outputToSave} in
 #'                 the \code{CacheRepo}
 writeFuture <- function(written, outputToSave, cacheRepo, userTags) {
+  counter <- 0
+  if (!file.exists(file.path(cacheRepo, "backpack.db"))) {
+    stop("That cacheRepo does not exist")
+  }
   while (written >= 0) {
     #future::plan(multiprocess)
-    saved <- #suppressWarnings(try(silent = TRUE,
+    saved <- #suppressWarnings(
+      try(#silent = TRUE,
       saveToLocalRepo(
         outputToSave,
         repoDir = cacheRepo,
@@ -1220,7 +1258,8 @@ writeFuture <- function(written, outputToSave, cacheRepo, userTags) {
         silent = TRUE,
         userTags = userTags
       )
-    #))
+    #)
+    )
 
     # This is for simultaneous write conflicts. SQLite on Windows can't handle them.
     written <- if (is(saved, "try-error")) {
@@ -1229,6 +1268,10 @@ writeFuture <- function(written, outputToSave, cacheRepo, userTags) {
     } else {
       -1
     }
+    counter <- counter + 1
+    if (isTRUE(startsWith(saved[1], "Error in checkDirectory")))
+      stop(saved)
+    if (counter > 10) stop("Can't write to cacheRepo")
   }
   return(saved)
 
