@@ -3,32 +3,72 @@
 #' Currently, this only deals with \code{\link[googledrive]{drive_download}},
 #' and \code{\link[utils]{download.file}}.
 #'
-#' @export
 #' @inheritParams prepInputs
-#' @include checksums.R
 #' @inheritParams extractFromArchive
 #' @param dlFun Optional "download function" name, such as \code{"raster::getData"}, which does
 #'              custom downloading, in addition to loading into R. Still experimental.
 #' @param ... Passed to \code{dlFun}. Still experimental.
 #' @param checksumFile A character string indicating the absolute path to the \code{CHECKSUMS.txt}
 #'                     file.
+#'
 #' @author Eliot McIntire
-downloadFile <- function(archive, targetFile, neededFiles, destinationPath, quick,
+#' @export
+#' @include checksums.R
+downloadFile <- function(archive, targetFile, neededFiles,
+                         destinationPath = getOption("reproducible.destinationPath"), quick,
                          checksumFile, dlFun = NULL,
-                         checkSums, url, needChecksums, overwrite = TRUE,
+                         checkSums, url, needChecksums, overwrite = getOption("reproducible.overwrite", TRUE),
                          purge = FALSE, ...) {
 
   if (!is.null(url) || !is.null(dlFun)) {
+    missingNeededFiles <- missingFiles(neededFiles, checkSums, targetFile)
 
-    if (is.null(neededFiles)) {
-      result <- unique(checkSums$result)
-    } else {
-      result <- checkSums[checkSums$expectedFile %in% neededFiles, ]$result
+    # if (is.null(neededFiles)) {
+    #   result <- unique(checkSums$result)
+    # } else {
+    #   result <- checkSums[checkSums$expectedFile %in% neededFiles, ]$result
+    # }
+    # if (length(result) == 0) result <- NA
+    #
+    # missingNeededFiles <- (!(all(compareNA(result, "OK")) && all(neededFiles %in% checkSums$expectedFile)) ||
+    #                          is.null(targetFile) || is.null(neededFiles))
+
+    if (missingNeededFiles) { # needed may be missing, but maybe can skip download b/c archive exists
+      if (!is.null(archive)) {
+        localArchivesExist <- file.exists(archive)
+        if (any(localArchivesExist)) {
+          filesInLocalArchives <- unique(basename(unlist(lapply(archive, .listFilesInArchive))))
+          if (all(neededFiles %in% filesInLocalArchives)) { # local archive has all files needed
+            extractedFromArchive <- extractFromArchive(archive = archive[localArchivesExist],
+                                                       destinationPath = destinationPath,
+                                                       neededFiles = neededFiles, checkSums = checkSums,
+                                                       needChecksums = needChecksums,
+                                                       checkSumFilePath = checksumFile,
+                                                       quick = quick)
+            checkSums <- if (!file.exists(checksumFile) || is.null(neededFiles)) {
+              needChecksums <- 1
+              .emptyChecksumsResult
+            } else {
+              Checksums(
+                files = file.path(destinationPath, basename(neededFiles)),
+                checksumFile = checksumFile,
+                path = destinationPath,
+                quickCheck = quick,
+                write = FALSE
+              )
+            }
+
+
+            # Check again, post extract ... If FALSE now, then it got it from local, already existing archive
+            missingNeededFiles <- missingFiles(neededFiles, checkSums, targetFile)
+            if (!missingNeededFiles) {
+              archive <- archive[localArchivesExist]
+            }
+          }
+        }
+      }
     }
-    if (length(result) == 0) result <- NA
 
-    missingNeededFiles <- (!(all(compareNA(result, "OK")) && all(neededFiles %in% checkSums$expectedFile)) ||
-                             is.null(targetFile) || is.null(neededFiles))
     if (missingNeededFiles) {
       if (needChecksums == 0) needChecksums <- 2 # use binary addition -- 1 is new file, 2 is append
     }
@@ -49,16 +89,16 @@ downloadFile <- function(archive, targetFile, neededFiles, destinationPath, quic
 
       # The download step
       downloadResults <- downloadRemote(url = url, archive = archive, # both url and fileToDownload must be NULL to skip downloading
-                     targetFile = targetFile, fileToDownload = fileToDownload,
-                     skipDownloadMsg = skipDownloadMsg,
-                     checkSums = checkSums,
-                     dlFun = dlFun,
-                     destinationPath = destinationPath,
-                     overwrite = overwrite,
-                     needChecksums = needChecksums, ...)
+                                        targetFile = targetFile, fileToDownload = fileToDownload,
+                                        skipDownloadMsg = skipDownloadMsg,
+                                        checkSums = checkSums,
+                                        dlFun = dlFun,
+                                        destinationPath = destinationPath,
+                                        overwrite = overwrite,
+                                        needChecksums = needChecksums, ...)
       if (file.exists(checksumFile)) {
         if (is.null(fileToDownload) || tryCatch(is.na(fileToDownload), warning = function(x) FALSE))  { # This is case where we didn't know what file to download, and only now
-                                        # do we know
+          # do we know
           fileToDownload <- downloadResults$destFile
         }
         if (!is.null(fileToDownload)) {
@@ -71,33 +111,29 @@ downloadFile <- function(archive, targetFile, neededFiles, destinationPath, quic
                 quickCheck = quick,
                 write = FALSE
               )
-            isOK <-
-              checkSums[checkSums$expectedFile %in% basename(fileToDownload) |
-                          checkSums$actualFile %in% basename(fileToDownload),]$result
+            isOK <- checkSums[checkSums$expectedFile %in% basename(fileToDownload) |
+                                checkSums$actualFile %in% basename(fileToDownload),]$result
             isOK <- isOK[!is.na(isOK)] == "OK"
-            if (length(isOK) > 0) {
+            if (length(isOK) > 0) { # This is length 0 if there are no entries in the Checksums
               if (!isTRUE(all(isOK))) {
                 if (purge > 0)  {
                   # This is case where we didn't know what file to download, and only now
                   # do we know
-                  checkSums <-
-                    .purge(checkSums = checkSums,
-                           purge = purge,
-                           url = fileToDownload)
+                  checkSums <- .purge(checkSums = checkSums,
+                                      purge = purge,
+                                      url = fileToDownload)
                   downloadResults$needChecksums <- 2
                 } else {
-                  tf <-
-                    tryCatch(
-                      basename(targetFile) %in% fileToDownload,
-                      error = function(x)
-                        FALSE
-                    )
-                  af <-
-                    tryCatch(
-                      basename(archive) %in% fileToDownload,
-                      error = function(x)
-                        FALSE
-                    )
+                  tf <- tryCatch(
+                    basename(targetFile) %in% fileToDownload,
+                    error = function(x)
+                      FALSE
+                  )
+                  af <- tryCatch(
+                    basename(archive) %in% fileToDownload,
+                    error = function(x)
+                      FALSE
+                  )
 
                   sc <- sys.calls()
                   piCall <- grep("^prepInputs", sc, value = TRUE)
@@ -137,9 +173,12 @@ downloadFile <- function(archive, targetFile, neededFiles, destinationPath, quic
           }
         }
       } # checksum file doesn't exist
-    } else { # not missing any files to download
+    } else {
+      # not missing any files to download
       fileAlreadyDownloaded <- if (is.null(archive[1])) {
-        archivePossibly <- setdiff(checkSums$expectedFile, neededFiles)
+        expectedFile <- checkSums[compareNA(checkSums$result, "OK"),]$expectedFile
+
+        archivePossibly <- setdiff(expectedFile, neededFiles)
         archivePossibly <- .isArchive(archivePossibly)
         if (!is.null(archivePossibly)) {
           archivePossibly
@@ -154,10 +193,17 @@ downloadFile <- function(archive, targetFile, neededFiles, destinationPath, quic
       downloadResults <- list(needChecksums = needChecksums,
                               destFile = file.path(destinationPath, basename(fileAlreadyDownloaded)))
       if (is.null(targetFile)) {
-        message("   Skipping download because all files listed in CHECKSUMS.txt file are present.",
+        message("   Skipping download because all needed files are listed in ",
+                "CHECKSUMS.txt file and are present.",
                 " If this is not correct, rerun prepInputs with purge = TRUE")
       } else {
-        message("  Skipping download: ", paste(neededFiles, collapse = ", ") ," already present")
+        if (exists("extractedFromArchive", inherits = FALSE)) {
+          message("  Skipping download: ", paste(neededFiles, collapse = ", ") ,
+                  ": extracted from local archive:\n    ",
+                  archive)
+        } else {
+          message("  Skipping download: ", paste(neededFiles, collapse = ", ") ," already present")
+        }
       }
     }
     archiveReturn <- if (is.null(archive)) {
@@ -165,8 +211,12 @@ downloadFile <- function(archive, targetFile, neededFiles, destinationPath, quic
     } else {
       archive
     }
-    if (!is.null(downloadResults$destFile))
-      neededFiles <- basename(unique(c(downloadResults$destFile, neededFiles)))
+
+
+    # This was commented out because of LandWeb -- removed b/c of this case:
+    #  have local archive, but not yet have the targetFile
+    # if (!is.null(downloadResults$destFile))
+    #   neededFiles <- unique(basename(c(downloadResults$destFile, neededFiles)))
   } else {
     downloadResults <- list(needChecksums = needChecksums, destFile = NULL)
     archiveReturn <- archive
@@ -207,25 +257,14 @@ downloadFile <- function(archive, targetFile, neededFiles, destinationPath, quic
 dlGoogle <- function(url, archive = NULL, targetFile = NULL,
                      checkSums, skipDownloadMsg, destinationPath,
                      overwrite, needChecksums) {
-  if (!is.null(googledrive::drive_token()))
-    googledrive::drive_auth() ## needed for use on e.g., rstudio-server
-  if (is.null(archive)) {
-    fileAttr <- googledrive::drive_get(googledrive::as_id(url))
-    archive <- .isArchive(fileAttr$name)
-    if (is.null(archive)) {
-      if (is.null(targetFile)) {
-        # make the guess
-        targetFile <- fileAttr$name
-      }
-      downloadFilename <- targetFile # override if the targetFile is not an archive
-    } else {
-      archive <- file.path(destinationPath, basename(archive))
-      downloadFilename <- archive
-    }
-  } else {
-    downloadFilename <- archive
-  }
-  destFile <- file.path(tempdir(), basename(downloadFilename))
+
+  downloadFilename <- assessGoogle(url = url, archive = archive,
+                                   targetFile = targetFile,
+                                   destinationPath = destinationPath)
+
+  #destFile <- tempfile(fileext = paste0(".",tools::file_ext(downloadFilename)))
+  destFile <- file.path(tempdir(), rndstr(len = 5), basename(downloadFilename))
+  checkPath(dirname(destFile), create = TRUE)
   if (!isTRUE(checkSums[checkSums$expectedFile ==  basename(destFile), ]$result == "OK")) {
     message("  Downloading from Google Drive.")
     googledrive::drive_download(googledrive::as_id(url), path = destFile,
@@ -240,7 +279,7 @@ dlGoogle <- function(url, archive = NULL, targetFile = NULL,
 #' Download file from generic source url
 #'
 #' @param url  The url (link) to the file.
-#' @param needChecksums TODO
+#' @param needChecksums Logical indicating whether to generate checksums.
 #'
 #' ## TODO: add overwrite arg to the function?
 #'
@@ -259,9 +298,9 @@ dlGeneric <- function(url, needChecksums) {
 
   ua <- httr::user_agent(getOption("reproducible.useragent"))
   request <- suppressWarnings(
-    ## TODO: GET is throving warnings
+    ## TODO: GET is throwing warnings
     httr::GET(url, ua, httr::progress(),
-              httr::write_disk(destFile, overwrite = TRUE)) ## TODO overwrite?
+              httr::write_disk(destFile, overwrite = TRUE)) ## TODO: overwrite?
   )
   httr::stop_for_status(request)
 
@@ -274,7 +313,7 @@ downloadRemote <- function(url, archive, targetFile, checkSums, dlFun = NULL,
 
   if (!is.null(url) || !is.null(dlFun)) { # if no url, no download
     #if (!is.null(fileToDownload)  ) { # don't need to download because no url --- but need a case
-      if (!isTRUE(tryCatch(is.na(fileToDownload), warning = function(x) FALSE)))  { # NA means archive already in hand
+      if (!isTRUE(tryCatch(is.na(fileToDownload), warning = function(x) FALSE)))  {# NA means archive already in hand
         if (!is.null(dlFun)) {
           dlFunName <- dlFun
           dlFun <- .extractFunction(dlFun)
@@ -308,6 +347,9 @@ downloadRemote <- function(url, archive, targetFile, checkSums, dlFun = NULL,
           } else {
             destFile <- file.path(destinationPath, targetFile)
           }
+
+          # some functions will load the object, not just download them, since we may not know
+          #   where the function actually downloaded the file, we save it as an RDS file
           if (needSave) {
             saveRDS(out, file = destFile)
           }
@@ -333,7 +375,24 @@ downloadRemote <- function(url, archive, targetFile, checkSums, dlFun = NULL,
         # if destinationPath is tempdir, then don't copy and remove
         if (!(identical(dirname(normPath(downloadResults$destFile)),
                         normPath(destinationPath)))) {
-          suppressWarnings(file.copy(downloadResults$destFile, destinationPath))
+          desiredPath <- file.path(destinationPath, basename(downloadResults$destFile))
+
+          desiredPathExists <- file.exists(desiredPath)
+          if (desiredPathExists && !isTRUE(overwrite)) {
+            stop(targetFile, " already exists at ", desiredPath, ". Use overwrite = TRUE?")
+          }
+          if (desiredPathExists) {
+            file.remove(desiredPath)
+          }
+
+          # Try hard link first -- the only type that R deeply recognizes
+          # if that fails, fall back to copying the file.
+          # NOTE: never use symlink because the original will be deleted.
+          warns <- capture_warnings(result <- file.link(downloadResults$destFile, desiredPath))
+          if (isFALSE(result)) {
+            result <- file.copy(downloadResults$destFile, desiredPath)
+          }
+
           suppressWarnings(file.remove(downloadResults$destFile))
           downloadResults$destFile <- file.path(destinationPath, basename(downloadResults$destFile))
         }
@@ -347,3 +406,46 @@ downloadRemote <- function(url, archive, targetFile, checkSums, dlFun = NULL,
   }
   downloadResults
 }
+
+missingFiles <- function(files, checkSums, targetFile) {
+  if (is.null(files)) {
+    result <- unique(checkSums$result)
+  } else {
+    result <- checkSums[checkSums$expectedFile %in% files, ]$result
+  }
+  if (length(result) == 0) result <- NA
+
+  (!(all(compareNA(result, "OK")) && all(files %in% checkSums$expectedFile)) ||
+      #                     is.null(targetFile) ||
+      is.null(files))
+}
+
+#' @importFrom googledrive as_id drive_get drive_token
+#' @importFrom quickPlot isRstudioServer
+assessGoogle <- function(url, archive = NULL, targetFile = NULL,
+                         destinationPath = getOption("reproducible.destinationPath")) {
+  if (isRstudioServer()) {
+    opts <- options(httr_oob_default = TRUE)
+    on.exit(options(opts))
+  }
+
+  if (!is.null(googledrive::drive_token()))
+    googledrive::drive_auth() ## needed for use on e.g., rstudio-server
+
+  if (is.null(archive)) {
+    fileAttr <- googledrive::drive_get(googledrive::as_id(url))
+    archive <- .isArchive(fileAttr$name)
+    if (is.null(archive)) {
+      if (is.null(targetFile)) {
+        # make the guess
+        targetFile <- fileAttr$name
+      }
+      downloadFilename <- targetFile # override if the targetFile is not an archive
+    } else {
+      archive <- file.path(destinationPath, basename(archive))
+      downloadFilename <- archive
+    }
+  } else {
+    downloadFilename <- archive
+  }
+}
