@@ -29,12 +29,13 @@
 #' normal, caching the return value at the point of the \code{Cache} call
 #' into the cache repository for later use.
 #'
-#' @name pipe2
+#' @inheritParams magrittr::`%>%`
+#'
 #' @aliases %>%
 #' @importFrom utils getFromNamespace
-#' @inheritParams magrittr::`%>%`
 #' @importFrom magrittr freduce
 #' @export
+#' @name pipe2
 #' @seealso pipe
 #' @rdname pipe2
 #' @examples
@@ -147,14 +148,16 @@
 #' that uses pipes (\code{Cache() \%C\% ... remaining pipes}.
 #' \code{This is still experimental; use with care}.
 #'
-#' @rdname pipe
-#' @name pipe
-#' @importFrom utils getFromNamespace
 #' @inheritParams magrittr::`%>%`
-#' @importFrom magrittr freduce
-#' @seealso pipe2
+#'
 #' @aliases %C%
 #' @export
+#' @importFrom utils getFromNamespace
+#' @importFrom magrittr freduce
+#' @name pipe
+#' @seealso pipe2
+#' @rdname pipe
+#'
 #' @examples
 #'
 #' # dontrun{ # these can't be automatically run due to package conflicts with magrittr
@@ -197,8 +200,7 @@
   whPipeCall <- unlist(lapply(mcs, function(elem) as.character(elem[[1]]) %in% c("%C%", "%>%")))
   # Take the first one, which will be one with the whole pipe sequence
   mc <- mcs[whPipeCall][[1]]
-  mc <- parse(text = gsub(deparse(mc), pattern = "%C%",
-                                    replacement = "%>%"))[[1]]
+  mc <- parse(text = gsub(deparse(mc), pattern = "%C%", replacement = "%>%"))[[1]]
   chain_parts <- getFromNamespace("split_chain", ns = "magrittr")(mc, env = env) # nolint
   pipes <- chain_parts[["pipes"]][-1]
   rhss <- chain_parts[["rhss"]][-1]
@@ -221,7 +223,6 @@
   if (getFromNamespace("is_placeholder", ns = "magrittr")(lhs)) {
     env[["_fseq"]]
   } else {
-
     # reproducible package code here until end of if statement
     cacheCall <- match.call(Cache, chain_parts[["lhs"]])
     cacheArgs <- lapply(cacheCall, function(x) x)
@@ -238,12 +239,12 @@
     result <- withVisible(do.call("Cache", args))
 
     if (getFromNamespace("is_compound_pipe", ns = "magrittr")(pipes[[1L]])) {
-      eval(call("<-", lhs, result[["value"]]), parent,
-           parent)
+      eval(call("<-", lhs, result[["value"]]), parent, parent)
     } else {
       if (result[["visible"]])
         result[["value"]]
-      else invisible(result[["value"]])
+      else
+        invisible(result[["value"]])
     }
   }
 }
